library flutter_intro;

import 'dart:async';

import 'package:flutter/material.dart';
import 'package:flutter_intro/delay_rendered_widget.dart';
import 'package:flutter_intro/intro_step.dart';
import 'package:flutter_intro/step_widget_params.dart';

export 'package:flutter_intro/intro_step.dart';
export 'package:flutter_intro/step_widget_params.dart';

class Intro {
  bool _removed = false;
  double _widgetWidth;
  double _widgetHeight;
  Offset _widgetOffset;
  OverlayEntry _overlayEntry;
  int _currentStepIndex = 0;
  Widget _stepWidget;
  final Color _maskColor = Colors.black.withOpacity(.6);
  final Duration _animationDuration = Duration(milliseconds: 300);
  final List<IntroStep> steps;
  final double padding = 8;
  Intro({this.steps}) : assert(steps.isNotEmpty);

  void _getWidgetInfo(GlobalKey globalKey) {
    RenderBox renderBox = globalKey.currentContext.findRenderObject();
    _widgetWidth = renderBox.size.width + padding;
    _widgetHeight = renderBox.size.height + padding;
    _widgetOffset = Offset(
      renderBox.localToGlobal(Offset.zero).dx - padding / 2,
      renderBox.localToGlobal(Offset.zero).dy - padding / 2,
    );
  }

  Widget _maskBuilder({
    @required double width,
    @required double height,
    BlendMode backgroundBlendMode,
    @required double left,
    @required double top,
    Widget child,
  }) {
    return AnimatedPositioned(
      duration: _animationDuration,
      child: AnimatedContainer(
        padding: EdgeInsets.all(padding / 2),
        decoration: BoxDecoration(
          color: Colors.white,
          backgroundBlendMode: backgroundBlendMode,
          borderRadius: BorderRadius.all(Radius.circular(4)),
        ),
        width: width,
        height: height,
        duration: _animationDuration,
        child: child,
      ),
      left: left,
      top: top,
    );
  }

  void _showOverlay(
    BuildContext context,
    GlobalKey globalKey,
  ) {
    Size screenSize = MediaQuery.of(context).size;
    _overlayEntry = new OverlayEntry(
      builder: (BuildContext context) {
        return DelayRenderedWidget(
          removed: _removed,
          childPersist: true,
          duration: _animationDuration,
          child: Material(
            color: Colors.transparent,
            child: Stack(
              children: [
                ColorFiltered(
                  colorFilter: ColorFilter.mode(
                    _maskColor,
                    BlendMode.srcOut,
                  ),
                  child: Stack(
                    children: [
                      _maskBuilder(
                        width: screenSize.width,
                        height: screenSize.height,
                        backgroundBlendMode: BlendMode.dstOut,
                        left: 0,
                        top: 0,
                      ),
                      _maskBuilder(
                        width: _widgetWidth,
                        height: _widgetHeight,
                        left: _widgetOffset.dx,
                        top: _widgetOffset.dy,
                      ),
                    ],
                  ),
                ),
                DelayRenderedWidget(
                  child: _stepWidget,
                ),
              ],
            ),
          ),
        );
      },
    );
    Overlay.of(context).insert(_overlayEntry);
  }

  void _onNext() {
    _currentStepIndex++;
    if (_currentStepIndex < steps.length) {
      _createStepWidget();
      _overlayEntry.markNeedsBuild();
    }
  }

  void _onPrev() {
    _currentStepIndex--;
    if (_currentStepIndex >= 0) {
      _createStepWidget();
      _overlayEntry.markNeedsBuild();
    }
  }

  void _createStepWidget() {
    _getWidgetInfo(steps[_currentStepIndex].key);

    var widgetBuilder = steps[_currentStepIndex].widgetBuilder;

    _stepWidget = widgetBuilder(StepWidgetParams(
      onNext: _currentStepIndex == steps.length - 1 ? null : _onNext,
      onPrev: _currentStepIndex == 0 ? null : _onPrev,
      offset: _widgetOffset,
      width: _widgetWidth,
      height: _widgetHeight,
      currentStepIndex: _currentStepIndex,
      stepCount: steps.length,
      onFinish: () {
        _removed = true;
        _overlayEntry.markNeedsBuild();
        Timer(_animationDuration, () {
          _overlayEntry.remove();
        });
      },
    ));
  }

  void start(BuildContext context) {
    _currentStepIndex = 0;
    _removed = false;
    GlobalKey currentKey = steps[_currentStepIndex].key;
    _createStepWidget();
    _showOverlay(
      context,
      currentKey,
    );
  }
}
